import sumpf
from matplotlib import pyplot
def system(excitation):
    distorted = 0.5 * excitation ** 3 - 0.6 * excitation ** 2 + 0.1 * excitation + 0.02
    highpass = sumpf.Chebyshev1Filter(cutoff_frequency=100.0, ripple=4.0, order=4, highpass=True)
    lowpass = sumpf.ButterworthFilter(cutoff_frequency=3000.0, order=2, highpass=False)
    filtered = distorted * highpass * lowpass
    shifted = filtered.shift(50)
    return shifted
sweep = sumpf.ExponentialSweep(start_frequency=20.0, stop_frequency=5000.0, interval=(4096, -4096), sampling_rate=48000, length=2 ** 16)
fade = sumpf.Fade(rise_interval=(0, 4096), fall_interval=(-4096, 1.0), sampling_rate=48000.0, length=2 ** 16)
excitation = sweep * fade
response = system(excitation)
inverse = sumpf.InverseExponentialSweep(start_frequency=20.0, stop_frequency=5000.0, interval=(4096, -4096), sampling_rate=48000, length=2 ** 16)
impulse_response = response.convolve(inverse)
harmonic1 = sweep.harmonic_impulse_response(impulse_response=impulse_response,
                                            harmonic=1)
harmonic2 = sweep.harmonic_impulse_response(impulse_response=impulse_response,
                                            harmonic=2)
harmonic3 = sweep.harmonic_impulse_response(impulse_response=impulse_response,
                                            harmonic=3)
harmonics = sumpf.Merge([harmonic1, harmonic2, harmonic3]).output()
transfer_function = harmonics.fourier_transform()
for magnitude, label in zip(transfer_function.magnitude(), transfer_function.labels()):   # doctest: +SKIP
    pyplot.plot(transfer_function.frequency_samples(), magnitude, label=label)            # doctest: +SKIP
pyplot.axvline(20.0, linestyle="--", color="r")                                           # doctest: +SKIP
pyplot.axvline(5000.0, linestyle="--", color="r")                                         # doctest: +SKIP
pyplot.axvline(sweep.minimum_frequency(), linestyle="--", color="k")                      # doctest: +SKIP
pyplot.axvline(sweep.maximum_frequency(), linestyle="--", color="k")                      # doctest: +SKIP
pyplot.axvline(100.0, linestyle="--", color="b")                                          # doctest: +SKIP
pyplot.axvline(3000.0, linestyle="--", color="b")                                         # doctest: +SKIP
pyplot.xlabel("frequency")                                                                # doctest: +SKIP
pyplot.ylabel("magnitude")                                                                # doctest: +SKIP
pyplot.loglog()                                                                           # doctest: +SKIP
pyplot.legend()                                                                           # doctest: +SKIP
pyplot.xlim(10.0, 12000.0)                                                                # doctest: +SKIP
pyplot.ylim(0.001, 10.0)                                                                  # doctest: +SKIP
pyplot.show()                                                                             # doctest: +SKIP